@@ -9,4 +9,4 @@ STATIC="-static"
 
 ${PREFIX}gcc -Os -Wall -I.. -c ssl_helper.c -o ssl_helper.o
 ${PREFIX}gcc $STATIC --start-group ssl_helper.o -lm ../src/.libs/libwolfssl.a --end-group -o ssl_helper
-${PREFIX}-strip ssl_helper
+${PREFIX}strip ssl_helper
